﻿using System;

//W3C log file format: http://www.w3.org/TR/WD-logfile.html
//cs-host extension:   http://www.microsoft.com/technet/prodtechnol/WindowsServer2003/Library/IIS/676400bc-8969-4aa7-851a-9319490a9bbb.mspx

namespace sdraw_w3c
{
    class Program
    {
        static DateTime LogDate; 

        static void Main(string[] args)
        {
            if(!GetParams(args)) {
                WriteHelp();
                Environment.ExitCode = 1;
                return;
            }

            var InFile = new JHSoftware.SimpleDNS.SDNSFileLib.RawLog(args[1]);
            var OutFile = new System.IO.StreamWriter(args[2], false, System.Text.Encoding.ASCII);

            var MyName = System.Reflection.Assembly.GetExecutingAssembly().GetName();
            OutFile.WriteLine("#Software: " + MyName.Name + ' ' + MyName.Version.ToString() );
            OutFile.WriteLine("#Version: 1.0"); //W3C log file format version
            OutFile.WriteLine("#Date: " + LogDate + " 00:00:00");
            OutFile.WriteLine("#Fields: date time c-ip cs-host x-qtype x-qclass x-rd x-opcode");

            foreach(var Request in InFile) 
            {
                OutFile.Write(LogDate.Add(Request.Time ).ToString("yyyy-MM-dd HH:mm:ss")); // date time
                OutFile.Write(' ');
                OutFile.Write(Request.FromIP.ToString()); //c-ip
                OutFile.Write(' ');
                OutFile.Write(Request.QName.ToString()); //cs-host
                OutFile.Write(' ');
                OutFile.Write(Request.QTypeName); //x-qtype
                OutFile.Write(' ');
                OutFile.Write(Request.QClassName); //x-qclass
                OutFile.Write(' ');
                OutFile.Write(Request.RD ? '1' : '0'); //x-rd
                OutFile.Write(' ');
                OutFile.Write(Request.OpCode.ToString()); //x-opcode
                OutFile.WriteLine();
            }
            OutFile.Close();
        }

        static bool GetParams(string[] args)
        {
            //command line params: log-date sdraw-input-file w3c-log-output-file
            if (args.Length != 3)
            {
                Console.WriteLine("*** Wrong number of arguments (<>3)");
                return false;
            }
            var ld = args[0];
            if (ld.Length != 10 | ld[4] != '-' | ld[7] != '-')
            {
                Console.WriteLine("*** Invalid log date argument (YYYY-MM-DD)");
                return false;
            }
            try 
            {
                LogDate = new DateTime(int.Parse(ld.Substring(0, 4)), int.Parse(ld.Substring(5, 2)), int.Parse(ld.Substring(8)));
            }
            catch
            {
                Console.WriteLine("*** Invalid log date argument (YYYY-MM-DD)");
                return false;
            }
            return true;
        }

        static void WriteHelp()
        {
            Console.WriteLine();
            Console.WriteLine("Converts a Simple DNS Plus raw log file (.sdraw) into a W3C Extended Log File format file");
            Console.WriteLine();
            Console.WriteLine("SDRAW-W3C <log-date> <raw-log-file> <w3c-log-file>");
            Console.WriteLine("  <log-date>      Date that the raw log file was recorded (YYYY-MM-DD format)");
            Console.WriteLine("  <raw-log-file>  Path/file name for the .sdraw file to convert (input)");
            Console.WriteLine("  <w3c-log-file>  Path/file name for the W3C log file (output)");
        }
    }
}

